/* VzImagination (C) 2018-2019 Stephane Charette <stephanecharette@gmail.com>
 * $Id: TestConstructors.cpp 2765 2019-04-30 22:11:22Z stephane $
 */

#include "TestUtil.hpp"


TEST(Constructor, Empty)
{
	// instantiating an object without any parms should result in an empty object
	vz::Imagination img;

	ASSERT_TRUE(img.empty());
}


TEST(Constructor, EmptyAndClear)
{
	vz::Imagination img1;
	vz::Imagination img2;

	ASSERT_TRUE(img1.empty());
	ASSERT_TRUE(img2.empty());
	ASSERT_EQ(img1, img2);

	// clearing an empty object should not make a difference
	img1.clear();

	ASSERT_TRUE(img1.empty());
	ASSERT_TRUE(img2.empty());
	ASSERT_EQ(img1, img2);
}


TEST(Constructor, ImageFile)
{
	test_images();

	vz::Imagination img1("b.jpg");
	ASSERT_FALSE(img1.empty());
	ASSERT_EQ(img1.size(), 1);

	vz::Imagination img2;
	ASSERT_TRUE(img2.empty());
	ASSERT_NE(img1, img2);

	vz::Imagination img3("r.jpg");
	ASSERT_FALSE(img3.empty());
	ASSERT_NE(img1, img3);

	vz::Imagination img4("g.jpg");
	ASSERT_FALSE(img4.empty());
	ASSERT_NE(img1, img4);

	vz::Imagination img5("k.jpg");
	ASSERT_FALSE(img5.empty());
	ASSERT_NE(img1, img5);
}


TEST(Constructor, ImageMat)
{
	test_images();

	vz::Imagination img1("r.jpg");
	ASSERT_FALSE(img1.empty());

	cv::Mat b(60, 80, CV_8UC3, cv::Scalar(0, 0, 255));
	vz::Imagination img2(b);
	ASSERT_EQ(img1, img2);

	vz::Imagination img3;
	img3.set_original(b);
	ASSERT_EQ(img3, img1);
	ASSERT_EQ(img3, img2);
}


TEST(Constructor, Copy)
{
	test_images();

	vz::Imagination img1("p.jpg");
	ASSERT_FALSE(img1.empty());
	ASSERT_EQ(img1.size(), 1);

	vz::Imagination img2(img1);
	ASSERT_FALSE(img2.empty());
	ASSERT_EQ(img2.size(), 1);
	ASSERT_EQ(img1, img2);
}


TEST(Constructor, Assignment)
{
	test_images();

	vz::Imagination img1("r.jpg");
	ASSERT_FALSE(img1.empty());
	ASSERT_EQ(img1.size(), 1);

	vz::Imagination img2("g.jpg");
	ASSERT_FALSE(img2.empty());
	ASSERT_EQ(img2.size(), 1);

	ASSERT_NE(img1, img2);
	img2 = img1;
	ASSERT_EQ(img1, img2);

	cv::Mat empty;
	img2 = empty;
	ASSERT_TRUE(img2.empty());
	ASSERT_NE(img1, img2);
}


TEST(Constructor, SetOriginal)
{
	vz::Imagination img("r.jpg");
	ASSERT_FALSE(img.empty());
	ASSERT_EQ(img.size(), 1);
	img.get(vz::Imagination::EMatType::kO2Invert);
	ASSERT_FALSE(img.empty());
	ASSERT_EQ(img.size(), 2);

	img.set_original("g.jpg");
	ASSERT_FALSE(img.empty());
	ASSERT_EQ(img.size(), 1);
	img.get(vz::Imagination::EMatType::kO2Invert);
	ASSERT_FALSE(img.empty());
	ASSERT_EQ(img.size(), 2);

	img.reset_using(vz::Imagination::EMatType::kO2SimpleColourBalance);
	ASSERT_FALSE(img.empty());
	ASSERT_EQ(img.size(), 1);
	img.get(vz::Imagination::EMatType::kO2Invert);
	ASSERT_FALSE(img.empty());
	ASSERT_EQ(img.size(), 2);

	img.reset();
	ASSERT_FALSE(img.empty());
	ASSERT_EQ(img.size(), 1);
}
