/* VzImagination (C) 2018-2019 Stephane Charette <stephanecharette@gmail.com>
 * $Id: TestType.cpp 2741 2019-04-09 17:04:20Z stephane $
 */

#include "TestUtil.hpp"


TEST(Type, Invalid)
{
	// the type 0 should always be considered uninitialized and map to "invalid"

	const int idx = 0;
	const vz::Imagination::EMatType type = static_cast<vz::Imagination::EMatType>(idx);

	ASSERT_EQ(type, vz::Imagination::EMatType::kInvalid);
}


TEST(Type, Unknown)
{
	// trying to get the name of an unknown image type should get back the name "unknown"

	const auto type = static_cast<vz::Imagination::EMatType>(-1);
	const auto name = vz::Imagination().to_string(type);
	ASSERT_EQ(name, "unknown");
}


TEST(Type, Names)
{
	// every image type should have a name

	for (int idx = -1; idx <= static_cast<int>(vz::Imagination::EMatType::kMax); idx++)
	{
		vz::Imagination::EMatType type = static_cast<vz::Imagination::EMatType>(idx);
		const auto name = vz::Imagination().to_string(type);

		std::cout << "idx " << idx << ": type is named \"" << name << "\"" << std::endl;

		ASSERT_NE(name, "");
	}
}


TEST(Type, NameUnique)
{
	// every name should be unique

	std::set<std::string> all_names;

	for (int idx = 0; idx <= static_cast<int>(vz::Imagination::EMatType::kMax); idx++)
	{
		vz::Imagination::EMatType type = static_cast<vz::Imagination::EMatType>(idx);
		const auto name = vz::Imagination().to_string(type);

		std::cout << "idx " << idx << ": checking that no other type is named \"" << name << "\"" << std::endl;

		ASSERT_EQ(all_names.count(name), 0);
		all_names.insert(name);
	}

	// there should be the same number of strings as there are image types (+1 since types are zero-based)
	ASSERT_EQ(all_names.size(), static_cast<size_t>(vz::Imagination::EMatType::kMax) + 1);
}
